@@ -36,15 +36,15 @@
 #ifndef META_MODELICA_BUILTIN_H_
 #define META_MODELICA_BUILTIN_H_
 
-#include "openmodelica.h"
-#include "util/omc_msvc.h"
+#include "../openmodelica.h"
+#include "../util/omc_msvc.h"
 
 #if defined(__cplusplus)
 extern "C" {
 #endif
 
 #include "meta_modelica_builtin_boxptr.h"
-#include "util/modelica_string_lit.h"
+#include "../util/modelica_string_lit.h"
 
 typedef modelica_metatype metamodelica_string;
 typedef const modelica_metatype metamodelica_string_const;
@@ -87,17 +87,15 @@ extern modelica_integer stringHashSdbm(metamodelica_string_const str);
 #define substring(X,Y,Z) boxptr_substring(threadData,X,mmc_mk_icon(Y),mmc_mk_icon(Z))
 extern modelica_metatype boxptr_substring(threadData_t *,metamodelica_string_const str, modelica_metatype start, modelica_metatype stop);
 
-#define System_stringHashDjb2Mod stringHashDjb2Mod
-#define boxptr_System_stringHashDjb2Mod boxptr_stringHashDjb2Mod
-#define boxvar_System_stringHashDjb2Mod boxvar_stringHashDjb2Mod
-
 extern modelica_metatype boxptr_stringHashDjb2Mod(threadData_t*,modelica_metatype v,modelica_metatype mod);
 
 /* List Operations */
 extern modelica_metatype listReverse(modelica_metatype);
 extern modelica_metatype listReverseInPlace(modelica_metatype);
 #define listSetRest(X,Y) boxptr_listSetRest(threadData,X,Y)
+#define listSetFirst(X,Y) boxptr_listSetFirst(threadData,X,Y)
 extern void boxptr_listSetRest(threadData_t*,modelica_metatype,modelica_metatype);
+extern void boxptr_listSetFirst(threadData_t*,modelica_metatype,modelica_metatype);
 extern modelica_boolean listMember(modelica_metatype, modelica_metatype);
 extern modelica_metatype listAppend(modelica_metatype,modelica_metatype);
 extern modelica_integer listLength(modelica_metatype);
@@ -129,6 +127,7 @@ extern modelica_metatype arrayList(modelica_metatype);
 extern modelica_metatype listArray(modelica_metatype);
 extern modelica_metatype arrayCopy(modelica_metatype);
 #define arrayGet(X,Y) nobox_arrayGet(threadData,X,Y)
+extern modelica_metatype listArrayLiteral(modelica_metatype);
 
 static inline modelica_metatype nobox_arrayGet(threadData_t *threadData,modelica_metatype arr,modelica_integer ix)
 {
@@ -143,7 +142,7 @@ static inline modelica_metatype arrayCreate(modelica_integer nelts, modelica_met
   if (nelts < 0) {
     MMC_THROW();
   } else {
-    void* arr = (struct mmc_struct*)mmc_mk_box_no_assign(nelts, MMC_ARRAY_TAG);
+    void* arr = (struct mmc_struct*)mmc_mk_box_no_assign(nelts, MMC_ARRAY_TAG, MMC_IS_IMMEDIATE(val));
     void **arrp = MMC_STRUCTDATA(arr);
     int i = 0;
     for(i=0; i<nelts; i++)
@@ -158,7 +157,7 @@ static inline modelica_metatype arrayCreateNoInit(modelica_integer nelts, modeli
   if (nelts < 0) {
     MMC_THROW();
   } else {
-    return (struct mmc_struct*)mmc_mk_box_no_assign(nelts, MMC_ARRAY_TAG);
+    return (struct mmc_struct*)mmc_mk_box_no_assign(nelts, MMC_ARRAY_TAG, 0);
   }
 }
 #define arrayGetNoBoundsChecking(arr,ix) (MMC_STRUCTDATA((arr))[(ix)-1])
@@ -180,7 +179,7 @@ static inline modelica_metatype boxptr_arrayUpdate(threadData_t *threadData,mode
 
 extern modelica_metatype boxptr_arrayUpdate(threadData_t *threadData,modelica_metatype, modelica_metatype, modelica_metatype);
 
-static inline modelica_metatype boxptr_arrayUpdateNoBoundsChecking(threadData_t *threadData,modelica_metatype arr, modelica_metatype i, modelica_metatype val)
+static inline modelica_metatype boxptr_arrayUpdateNoBoundsChecking(threadData_t *threadData __attribute__((unused)), modelica_metatype arr, modelica_metatype i, modelica_metatype val)
 {
   int ix = mmc_unbox_integer(i);
   MMC_STRUCTDATA(arr)[ix-1] = val;
@@ -215,6 +214,7 @@ extern modelica_integer intMaxLit(void);
 
 extern modelica_boolean setStackOverflowSignal(modelica_boolean);
 extern metamodelica_string referenceDebugString(modelica_metatype fnptr);
+extern metamodelica_string referencePointerString(modelica_metatype ptr);
 
 #include "meta_modelica_builtin_boxvar.h"
 
