@@ -41,21 +41,21 @@
 #include "simulation_data.h"
 #include "simulation/simulation_info_json.h"
 #include "util/omc_error.h"
+#include "util/parallel_helper.h"
+#include "omc_math.h"
 #include "util/varinfo.h"
 #include "model_help.h"
 
 #include "linearSystem.h"
 #include "linearSolverKlu.h"
 
-
 static void printMatrixCSC(int* Ap, int* Ai, double* Ax, int n);
 static void printMatrixCSR(int* Ap, int* Ai, double* Ax, int n);
 
 /*! \fn allocate memory for linear system solver Klu
  *
  */
-int
-allocateKluData(int n_row, int n_col, int nz, void** voiddata)
+int allocateKluData(int n_row, int n_col, int nz, void** voiddata)
 {
   DATA_KLU* data = (DATA_KLU*) malloc(sizeof(DATA_KLU));
   assertStreamPrint(NULL, 0 != data, "Could not allocate data for linear solver Klu.");
@@ -68,7 +68,6 @@ allocateKluData(int n_row, int n_col, int nz, void** voiddata)
   data->nnz = nz;
 
   data->Ap = (int*) calloc((n_row+1),sizeof(int));
-
   data->Ai = (int*) calloc(nz,sizeof(int));
   data->Ax = (double*) calloc(nz,sizeof(double));
   data->work = (double*) calloc(n_col,sizeof(double));
@@ -85,8 +84,7 @@ allocateKluData(int n_row, int n_col, int nz, void** voiddata)
 /*! \fn free memory for linear system solver Klu
  *
  */
-int
-freeKluData(void **voiddata)
+int freeKluData(void **voiddata)
 {
   TRACE_PUSH
 
@@ -97,6 +95,7 @@ freeKluData(void **voiddata)
   free(data->Ax);
   free(data->work);
 
+
   if(data->symbolic)
     klu_free_symbolic(&data->symbolic, &data->common);
   if(data->numeric)
@@ -116,22 +115,34 @@ freeKluData(void **voiddata)
  *  \author wbraun
  *
  */
-static
-int getAnalyticalJacobian(DATA* data, threadData_t *threadData, int sysNumber)
+static int getAnalyticalJacobian(DATA* data, threadData_t *threadData,
+                                 int sysNumber)
 {
   int i,ii,j,k,l;
+
   LINEAR_SYSTEM_DATA* systemData = &(((DATA*)data)->simulationInfo->linearSystemData[sysNumber]);
 
   const int index = systemData->jacobianIndex;
-  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[systemData->jacobianIndex]);
-  ANALYTIC_JACOBIAN* parentJacobian = systemData->parentJacobian;
+  ANALYTIC_JACOBIAN* jacobian = systemData->parDynamicData[omc_get_thread_num()].jacobian;
+  ANALYTIC_JACOBIAN* parentJacobian = systemData->parDynamicData[omc_get_thread_num()].parentJacobian;
 
   int nth = 0;
-  int nnz = jacobian->sparsePattern.numberOfNoneZeros;
+  int nnz = jacobian->sparsePattern->numberOfNoneZeros;
 
-  for(i=0; i < jacobian->sizeRows; i++)
+  if (jacobian->constantEqns != NULL) {
+    jacobian->constantEqns(data, threadData, jacobian, parentJacobian);
+  }
+
+  for(i=0; i < jacobian->sparsePattern->maxColors; i++)
   {
-    jacobian->seedVars[i] = 1;
+    /* activate seed variable for the corresponding color */
+    for(ii=0; ii < jacobian->sizeCols; ii++)
+    {
+      if(jacobian->sparsePattern->colorCols[ii]-1 == i)
+      {
+        jacobian->seedVars[ii] = 1;
+      }
+    }
 
     ((systemData->analyticalJacobianColumn))(data, threadData, jacobian, parentJacobian);
 
@@ -139,21 +150,18 @@ int getAnalyticalJacobian(DATA* data, threadData_t *threadData, int sysNumber)
     {
       if(jacobian->seedVars[j] == 1)
       {
-        ii = jacobian->sparsePattern.leadindex[j];
-        while(ii < jacobian->sparsePattern.leadindex[j+1])
+        nth = jacobian->sparsePattern->leadindex[j];
+        while(nth < jacobian->sparsePattern->leadindex[j+1])
         {
-          l  = jacobian->sparsePattern.index[ii];
-          systemData->setAElement(i, l, -jacobian->resultVars[l], nth, (void*) systemData, threadData);
+          l  = jacobian->sparsePattern->index[nth];
+          systemData->setAElement(j, l, -jacobian->resultVars[l], nth, (void*) systemData, threadData);
           nth++;
-          ii++;
-        };
+        }
+        /* de-activate seed variable for the corresponding color */
+        jacobian->seedVars[j] = 0;
       }
-    };
-
-    /* de-activate seed variable for the corresponding color */
-    jacobian->seedVars[i] = 0;
+    }
   }
-
   return 0;
 }
 
@@ -176,12 +184,12 @@ static int residual_wrapper(double* x, double* f, void** data, int sysNumber)
  *
  * author: wbraun
  */
-int
-solveKlu(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
+int solveKlu(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
 {
   void *dataAndThreadData[2] = {data, threadData};
   LINEAR_SYSTEM_DATA* systemData = &(data->simulationInfo->linearSystemData[sysNumber]);
-  DATA_KLU* solverData = (DATA_KLU*)systemData->solverData[0];
+  DATA_KLU* solverData = (DATA_KLU*)systemData->parDynamicData[omc_get_thread_num()].solverData[0];
+  _omc_scalar residualNorm = 0;
 
   int i, j, status = 0, success = 0, n = systemData->size, eqSystemNumber = systemData->equationIndex, indexes[2] = {1,eqSystemNumber};
   double tmpJacEvalTime;
@@ -218,7 +226,8 @@ solveKlu(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
 
     /* calculate vector b (rhs) */
     memcpy(solverData->work, aux_x, sizeof(double)*solverData->n_row);
-    residual_wrapper(solverData->work, systemData->b, dataAndThreadData, sysNumber);
+
+  residual_wrapper(solverData->work, systemData->parDynamicData[omc_get_thread_num()].b, dataAndThreadData, sysNumber);
   }
   tmpJacEvalTime = rt_ext_tp_tock(&(solverData->timeClock));
   systemData->jacobianTime += tmpJacEvalTime;
@@ -241,11 +250,13 @@ solveKlu(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
     messageClose(LOG_LS_V);
 
     for (i=0; i<solverData->n_row; i++)
-      infoStreamPrint(LOG_LS_V, 0, "b[%d] = %e", i, systemData->b[i]);
+    {
+      // ToDo Rework stream prints like this one to work in parallel regions
+      infoStreamPrint(LOG_LS_V, 0, "b[%d] = %e", i, systemData->parDynamicData[omc_get_thread_num()].b[i]);
+    }
   }
   rt_ext_tp_tick(&(solverData->timeClock));
 
-
   /* symbolic pre-ordering of A to reduce fill-in of L and U */
   if (0 == solverData->numberSolving)
   {
@@ -277,11 +288,11 @@ solveKlu(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
 
   if (0 == solverData->common.status){
     if (1 == systemData->method){
-      if (klu_solve(solverData->symbolic, solverData->numeric, solverData->n_col, 1, systemData->b, &solverData->common)){
+      if (klu_solve(solverData->symbolic, solverData->numeric, solverData->n_col, 1, systemData->parDynamicData[omc_get_thread_num()].b, &solverData->common)){
         success = 1;
       }
     } else {
-      if (klu_tsolve(solverData->symbolic, solverData->numeric, solverData->n_col, 1, systemData->b, &solverData->common)){
+      if (klu_tsolve(solverData->symbolic, solverData->numeric, solverData->n_col, 1, systemData->parDynamicData[omc_get_thread_num()].b, &solverData->common)){
         success = 1;
       }
     }
@@ -295,18 +306,30 @@ solveKlu(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
     if (1 == systemData->method){
       /* take the solution */
       for(i = 0; i < solverData->n_row; ++i)
-        aux_x[i] += systemData->b[i];
+        aux_x[i] += systemData->parDynamicData[omc_get_thread_num()].b[i];
 
       /* update inner equations */
       residual_wrapper(aux_x, solverData->work, dataAndThreadData, sysNumber);
+      residualNorm = _omc_gen_euclideanVectorNorm(solverData->work, solverData->n_row);
+
+      if ((isnan(residualNorm)) || (residualNorm>1e-4)){
+        warningStreamPrint(LOG_LS, 0,
+            "Failed to solve linear system of equations (no. %d) at time %f. Residual norm is %.15g.",
+            (int)systemData->equationIndex, data->localData[0]->timeValue, residualNorm);
+        success = 0;
+      }
     } else {
       /* the solution is automatically in x */
-      memcpy(aux_x, systemData->b, sizeof(double)*systemData->size);
+      memcpy(aux_x, systemData->parDynamicData[omc_get_thread_num()].b, sizeof(double)*systemData->size);
     }
 
     if (ACTIVE_STREAM(LOG_LS_V))
     {
-      infoStreamPrint(LOG_LS_V, 1, "Solution x:");
+      if (1 == systemData->method) {
+        infoStreamPrint(LOG_LS_V, 1, "Residual Norm %.15g of solution x:", residualNorm);
+      } else {
+        infoStreamPrint(LOG_LS_V, 1, "Solution x:");
+      }
       infoStreamPrint(LOG_LS_V, 0, "System %d numVars %d.", eqSystemNumber, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).numVar);
 
       for(i = 0; i < systemData->size; ++i)
