@@ -30,7 +30,10 @@
 
 
 #include "boolean_array.h"
-#include "gc/omc_gc.h"
+#include "../gc/omc_gc.h"
+#include "omc_error.h"
+#include "generic_array.h"
+
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -88,17 +91,7 @@ void alloc_boolean_array_data(boolean_array_t* a)
 
 void copy_boolean_array_data(const boolean_array_t source, boolean_array_t *dest)
 {
-    size_t i, nr_of_elements;
-
-    assert(base_array_ok(&source));
-    assert(base_array_ok(dest));
-    assert(base_array_shape_eq(&source, dest));
-
-    nr_of_elements = base_array_nr_of_elements(source);
-
-    for(i = 0; i < nr_of_elements; ++i) {
-        boolean_set(dest, i, boolean_get(source, i));
-    }
+    boolean_array_copy_data(source,*dest);
 }
 
 void and_boolean_array(const boolean_array_t *source1, const boolean_array_t *source2, boolean_array_t *dest)
@@ -175,9 +168,7 @@ void copy_boolean_array_data_mem(const boolean_array_t source, modelica_boolean
 
 void copy_boolean_array(const boolean_array_t source, boolean_array_t *dest)
 {
-    clone_base_array_spec(&source, dest);
-    alloc_boolean_array_data(dest);
-    copy_boolean_array_data(source,dest);
+    boolean_array_alloc_copy(source,*dest);
 }
 
 /*
@@ -306,44 +297,20 @@ void simple_indexed_assign_boolean_array2(const boolean_array_t* source,
 void indexed_assign_boolean_array(const boolean_array_t source, boolean_array_t* dest,
                                   const index_spec_t* dest_spec)
 {
-    _index_t* idx_vec1;
-    _index_t* idx_size;
-    int i,j;
-
-    assert(base_array_ok(&source));
-    assert(base_array_ok(dest));
-    assert(index_spec_ok(dest_spec));
-    assert(index_spec_fit_base_array(dest_spec, dest));
-    for(i = 0,j = 0; i < dest_spec->ndims; ++i) {
-        if(dest_spec->dim_size[i] != 0) {
-            ++j;
-        }
-    }
-    assert(j == source.ndims);
-
-    idx_vec1 = size_alloc(dest->ndims);
-    idx_size = size_alloc(dest_spec->ndims);
-
-    for(i = 0; i < dest_spec->ndims; ++i) {
-        idx_vec1[i] = 0;
-
-        if(dest_spec->index[i] != NULL) { /* is 'S' or 'A' */
-            idx_size[i] = imax(dest_spec->dim_size[i],1);
-        } else { /* is 'W' */
-            idx_size[i] = dest->dim_size[i];
-        }
-    }
+    _index_t *idx_vec1, *idx_size;
+    int j;
+    indexed_assign_base_array_size_alloc(&source, dest, dest_spec, &idx_vec1, &idx_size);
 
     j = 0;
     do {
         boolean_set(dest,
-                    calc_base_index_spec(dest->ndims, idx_vec1, dest, dest_spec),
-                    boolean_get(source, j));
+                 calc_base_index_spec(dest->ndims, idx_vec1, dest, dest_spec),
+                 boolean_get(source, j));
         j++;
 
     } while(0 == next_index(dest_spec->ndims, idx_vec1, idx_size));
 
-    assert(j == base_array_nr_of_elements(source));
+    omc_assert_macro(j == base_array_nr_of_elements(source));
 }
 
 /*
@@ -428,33 +395,7 @@ void index_alloc_boolean_array(const boolean_array_t* source,
                                const index_spec_t* source_spec,
                                boolean_array_t* dest)
 {
-    int i;
-    int j;
-
-    assert(base_array_ok(source));
-    assert(index_spec_ok(source_spec));
-    assert(index_spec_fit_base_array(source_spec, source));
-
-    for(i = 0, j = 0; i < source_spec->ndims; ++i) {
-         if(source_spec->dim_size[i] != 0) { /* is 'W' or 'A' */
-           ++j;
-         }
-    }
-    dest->ndims = j;
-    dest->dim_size = size_alloc(dest->ndims);
-
-    for(i = 0, j = 0; i < source_spec->ndims; ++i) {
-        if(source_spec->dim_size[i] != 0) { /* is 'W' or 'A' */
-            if(source_spec->index[i] != NULL) { /* is 'A' */
-                dest->dim_size[j] = source_spec->dim_size[i];
-            } else { /* is 'W' */
-                dest->dim_size[j] = source->dim_size[i];
-            }
-
-            ++j;
-        }
-    }
-
+    index_alloc_base_array_size(source, source_spec, dest);
     alloc_boolean_array_data(dest);
     index_boolean_array(source, source_spec, dest);
 }
@@ -571,22 +512,6 @@ void array_alloc_boolean_array(boolean_array_t* dest, int n,
     free(elts);
 }
 
-void array_scalar_boolean_array(boolean_array_t* dest, int n,
-                                m_boolean first, ...)
-{
-    int i;
-    va_list ap;
-    assert(base_array_ok(dest));
-    assert(dest->ndims == 1);
-    assert(dest->dim_size[0] == n);
-    put_boolean_element(first, 0, dest);
-    va_start(ap,first);
-    for(i = 0; i < n; ++i) {
-        put_boolean_element((m_boolean) va_arg(ap, int),i,dest);
-    }
-    va_end(ap);
-}
-
 /* array_alloc_scalar_boolean_array
  *
  * Creates(incl allocation) an array from scalar elements.
@@ -987,3 +912,33 @@ void fill_alloc_boolean_array(boolean_array_t* dest, modelica_boolean value, int
         boolean_set(dest, i, value);
     }
 }
+
+modelica_boolean min_boolean_array(const boolean_array_t a)
+{
+  size_t nr_of_elements;
+
+  omc_assert_macro(base_array_ok(&a));
+  nr_of_elements = base_array_nr_of_elements(a);
+
+  size_t i;
+  for (i = 0; i < nr_of_elements; ++i) {
+    if (!boolean_get(a, i)) return 0;
+  }
+
+  return 1;
+}
+
+modelica_boolean max_boolean_array(const boolean_array_t a)
+{
+  size_t nr_of_elements;
+
+  omc_assert_macro(base_array_ok(&a));
+  nr_of_elements = base_array_nr_of_elements(a);
+
+  size_t i;
+  for (i = 0; i < nr_of_elements; ++i) {
+    if (boolean_get(a, i)) return 1;
+  }
+
+  return 0;
+}
