@@ -31,8 +31,9 @@
 
 #include "integer_array.h"
 #include "index_spec.h"
-#include "gc/omc_gc.h"
+#include "../gc/omc_gc.h"
 #include "division.h"
+#include "generic_array.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,7 +41,7 @@
 #include <math.h>
 
 #include "omc_error.h"
-#include "meta/meta_modelica.h"
+#include "../meta/meta_modelica.h"
 
 static OMC_INLINE modelica_integer *integer_ptrget(const integer_array_t *a, size_t i)
 {
@@ -93,17 +94,7 @@ void alloc_integer_array_data(integer_array_t* a)
 
 void copy_integer_array_data(const integer_array_t source, integer_array_t* dest)
 {
-    size_t i, nr_of_elements;
-
-    omc_assert_macro(base_array_ok(&source));
-    omc_assert_macro(base_array_ok(dest));
-    omc_assert_macro(base_array_shape_eq(&source, dest));
-
-    nr_of_elements = base_array_nr_of_elements(source);
-
-    for(i = 0; i < nr_of_elements; ++i) {
-        integer_set(dest, i, integer_get(source, i));
-    }
+    integer_array_copy_data(source,*dest);
 }
 
 void copy_integer_array_data_mem(const integer_array_t source,
@@ -122,9 +113,7 @@ void copy_integer_array_data_mem(const integer_array_t source,
 
 void copy_integer_array(const integer_array_t source, integer_array_t *dest)
 {
-    clone_base_array_spec(&source, dest);
-    alloc_integer_array_data(dest);
-    copy_integer_array_data(*&source, dest);
+    integer_array_alloc_copy(source,*dest);
 }
 
 static modelica_integer integer_le(modelica_integer x, modelica_integer y)
@@ -547,6 +536,9 @@ modelica_integer* integer_array_element_addr1(const integer_array_t * source,
 
 modelica_integer* integer_array_element_addr2(const integer_array_t * source,int ndims,int dim1,int dim2)
 {
+    if (dim1 < 1 || dim1 > source->dim_size[0] || dim2 < 1 || dim2 > source->dim_size[1]) {
+      throwStreamPrint(NULL, "integer_array_element_addr2: array has dimensions [%ld,%ld], got subscripts [%d,%d]", source->dim_size[1], source->dim_size[2], dim1, dim2);
+    }
     return integer_ptrget(source, ((dim1 - 1) * source->dim_size[1]) + (dim2 - 1));
 }
 
@@ -1388,12 +1380,28 @@ void identity_alloc_integer_array(int n,integer_array_t* dest)
     identity_integer_array(n,dest);
 }
 
-void diagonal_integer_array(const integer_array_t * v,integer_array_t* dest)
+static void diagonal_integer_array_impl(const integer_array_t *v, integer_array_t* dest)
 {
     size_t i;
     size_t j;
     size_t n;
 
+    n = v->dim_size[0];
+
+    for(i = 0; i < (n * n); ++i) {
+        integer_set(dest, i, 0);
+    }
+    j = 0;
+    for(i = 0; i < n; ++i) {
+        integer_set(dest, j, integer_get(*v, i));
+        j += n + 1;
+    }
+}
+
+void diagonal_integer_array(const integer_array_t * v,integer_array_t* dest)
+{
+    size_t n;
+
     /* Assert that v is a vector */
     omc_assert_macro(v->ndims == 1);
 
@@ -1402,35 +1410,20 @@ void diagonal_integer_array(const integer_array_t * v,integer_array_t* dest)
     omc_assert_macro(dest->ndims == 2);
     omc_assert_macro((dest->dim_size[0] == n) && (dest->dim_size[1] == n));
 
-    for(i = 0; i < (n * n); ++i) {
-        integer_set(dest, i, 0);
-    }
-    j = 0;
-    for(i = 0; i < n; ++i) {
-        integer_set(dest, j, integer_get(*v, i));
-        j += n+1;
-    }
+    diagonal_integer_array_impl(v, dest);
 }
 
-void diagonal_alloc_integer_array(integer_array_t* dest, int ndims, ...)
+void diagonal_alloc_integer_array(const integer_array_t* v, integer_array_t* dest)
 {
-    size_t i;
-    size_t j;
-    va_list ap;
+    size_t n;
 
-    alloc_integer_array(dest,2,ndims,ndims);
+    /* Assert that v is a vector */
+    omc_assert_macro(v->ndims == 1);
 
-    for(i = 0; i < (ndims * ndims); ++i) {
-        integer_set(dest, i, 0);
-    }
-
-    va_start(ap,ndims);
-    j = 0;
-    for(i = 0; i < ndims; ++i) {
-        integer_set(dest, j, va_arg(ap, modelica_integer));
-        j += ndims+1;
-    }
-    va_end(ap);
+    /* Allocate a n*n matrix and fill it. */
+    n = v->dim_size[0];
+    alloc_integer_array(dest, 2, n, n);
+    diagonal_integer_array_impl(v, dest);
 }
 
 void fill_integer_array(integer_array_t* dest,modelica_integer s)
@@ -1560,13 +1553,13 @@ void symmetric_integer_array(const integer_array_t * a,integer_array_t* dest)
 
 /* integer_array_make_index_array
  *
- * Creates an integer array if indices to be used by e.g.
+ * Creates an integer array of indices to be used by e.g.
  ** create_index_spec defined in index_spec.c
  */
 
-_index_t* integer_array_make_index_array(const integer_array_t *arr)
+_index_t* integer_array_make_index_array(const integer_array_t arr)
 {
-    return arr->data;
+    return arr.data;
 }
 
 /* Converts the elements of an integer_array to int and packs them. I.e. if the
@@ -1599,6 +1592,56 @@ void unpack_integer_array(integer_array_t *a)
   }
 }
 
+/* Returns a modelica_integer array that can be treated as an int array. If the
+ * size of int and modelica_integer is the same this means simply returning the
+ * given array, but if int is smaller than modelica_integer a new array is
+ * allocated and filled with the data from given array as if it was an int array.
+ *
+ * I.e. if int is 32 bit and modelica_integer is 64 bit then the data will be
+ * packed into the first half of the new array.
+ *
+ * The case where int is larger than modelica_integer is not implemented. */
+void pack_alloc_integer_array(integer_array_t *a, integer_array_t *dest)
+{
+  if (sizeof(int) == sizeof(modelica_integer)) {
+    *dest = *a;
+  } else {
+    /* We only handle the case where int is smaller than modelica_integer. */
+    omc_assert_macro(sizeof(int) < sizeof(modelica_integer));
+
+    /* Allocate a new array. */
+    clone_integer_array_spec(a, dest);
+    alloc_integer_array_data(dest);
+
+    /* Pretend that the new array is an int array and fill it with the values
+     * from the given array. */
+    int *int_data = (int*)dest->data;
+    long i;
+    size_t n = base_array_nr_of_elements(*a);
+
+    for (i = 0; i < n; ++i) {
+      int_data[i] = (int)integer_get(*a, i);
+    }
+  }
+}
+
+/* Unpacks an integer_array that was packed with pack_integer_array into the
+ * destination array. If packing hasn't been done, i.e. if the size of int and
+ * modelica_integer is the same, then the function does nothing since both the
+ * source and destination is assumed to be the same array. */
+void unpack_copy_integer_array(const integer_array_t *a, integer_array_t *dest)
+{
+  if(sizeof(int) != sizeof(modelica_integer)) {
+    long i;
+    const int * int_data = (const int*)a->data;
+    long n = (long)base_array_nr_of_elements(*a);
+
+    for(i = n - 1; i >= 0; --i) {
+      integer_set(dest, i, int_data[i]);
+    }
+  }
+}
+
 void convert_alloc_integer_array_to_f77(const integer_array_t * a,
                                         integer_array_t* dest)
 {
