@@ -21,7 +21,10 @@ struct SimSettings
   LogSettings logSettings;
   bool nonLinearSolverContinueOnError;
   int solverThreads;
-  OutputFormat outputFomrat;
+  OutputFormat outputFormat;
+  EmitResults emitResults;
+  string inputPath;
+  string outputPath;
 };
 
 /**
@@ -36,12 +39,11 @@ public:
   virtual weak_ptr<IMixedSystem> LoadSystem(string modelLib,string modelKey) = 0;
   virtual weak_ptr<IMixedSystem> LoadModelicaSystem(PATH modelica_path,string modelKey) = 0;
   virtual void Start(SimSettings simsettings, string modelKey)=0;
-
-  virtual void StartVxWorks(SimSettings simsettings,string modelKey) = 0;
   virtual shared_ptr<IMixedSystem> getSystem(string modelname) = 0;
   virtual  shared_ptr<ISimObjects> getSimObjects() = 0;
-  virtual void calcOneStep() = 0;
-
+  virtual void initialize(SimSettings simsettings, string modelKey, double timeout)=0;
+  virtual void StartReduceDAE(SimSettings simsettings,string modelPath, string modelKey, bool loadMSL, bool loadPackage)=0;
+  virtual void runReducedSimulation()=0;
   /**
    *    Stops the simulation
    */
