@@ -70,9 +70,15 @@ namespace oms
     oms_status_enu_t rename(const ComRef& cref);
     oms_status_enu_t list(const ComRef& cref, char** contents);
     oms_status_enu_t addSystem(const ComRef& cref, oms_system_enu_t type);
-    oms_status_enu_t exportToSSD(pugi::xml_node& node) const;
+    oms_status_enu_t exportToSSD(pugi::xml_node& node, pugi::xml_node& ssvNode) const;
+    oms_status_enu_t exportSnapshot(const ComRef& cref, char** contents);
+    oms_status_enu_t exportSSVTemplate(const ComRef& cref, const std::string& filename);
     oms_status_enu_t importFromSSD(const pugi::xml_node& node);
+    oms_status_enu_t importSnapshot(const char* snapshot);
     oms_status_enu_t exportToFile(const std::string& filename) const;
+    oms_system_enu_t getSystemType(const pugi::xml_node& node, const std::string& sspVersion);
+    oms_system_enu_t getSystemTypeHelper(const pugi::xml_node& node, const std::string& sspVersion);
+    oms_status_enu_t updateParameterBindingsToSSD(pugi::xml_node& node, pugi::xml_node& ssvNode, bool isTopSystemOrModel) const;
     void copyResources(bool copy_resources) {this->copy_resources = copy_resources;}
     bool copyResources() {return copy_resources;}
 
@@ -97,9 +103,12 @@ namespace oms
     oms_status_enu_t setLoggingInterval(double loggingInterval);
     double getLoggingInterval() const {return loggingInterval;}
     oms_status_enu_t setResultFile(const std::string& filename, int bufferSize);
+    oms_status_enu_t getResultFile(char** filename, int* bufferSize);
     oms_status_enu_t emit(double time, bool force=false, bool* emitted=NULL);
     oms_status_enu_t addSignalsToResults(const char* regex);
     oms_status_enu_t removeSignalsFromResults(const char* regex);
+    oms_status_enu_t setSignalFilter(const char* regex);
+    oms_status_enu_t getSignalFilter(char** regex);
 
     oms_status_enu_t cancelSimulation_asynchronous();
     bool cancelSimulation() const {return cancelSim;}
@@ -114,6 +123,8 @@ namespace oms
 
     oms_status_enu_t loadSnapshot(const char* snapshot);
 
+    pugi::xml_node getSnapshot() {return snapShot;}
+
   private:
     Model(const ComRef& cref, const std::string& tempDir);
 
@@ -144,6 +155,10 @@ namespace oms
     std::string resultFilename;             ///< default <name>_res.mat
     Clock clock;
 
+    std::string signalFilter = "";  ///< default set to empty
+
+    pugi::xml_node snapShot; ///< top level snapshot node which contains ssd, ssv and ssm as child nodes
+
     bool cancelSim;
     bool isolatedFMU = false;
 
